@@ -20,6 +20,7 @@ void main(int argc, char * argv[])
  int i;
  int flags = 0;
  
+  struct in_addr ip;
  while((i=getopt(argc, argv, "dps"))!=-1)
   switch(i)
   {
@@ -39,11 +40,11 @@ void main(int argc, char * argv[])
 	}
 
  globals = hg_init(argv[optind], flags);
- m = hg_next_host(globals,NULL);
+ m = hg_next_host(globals,&ip);
  while(m)
  {
-  printf("%s\n", m);
-  m = hg_next_host(globals,NULL);
+  printf("%s (%s)\n", m, inet_ntoa(ip));
+  m = hg_next_host(globals,&ip);
  }
  hg_cleanup(globals);
 }
