@@ -6,7 +6,7 @@
  *
  *       Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *    $Id: hlst.c,v 1.14 1999/11/27 19:51:40 renaud Exp $ 
+ *    $Id: hlst.c,v 1.16 2000/05/05 17:47:53 renaud Exp $ 
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -87,7 +87,7 @@ static const hash_defs hints [] = {
   {269,  97},
   {509, 101},
   {577, 107},
-  0
+  {0,0}
 };
 
 /* ------------------------------------------------------------------------- *
@@ -158,8 +158,17 @@ create_hlst
     estimated_size_hint = DEFAULT_BUCKETS ;
 
   /* find appropriate list size, will stop at the last entry */
-  while (hd [1].mod && hd [1].mod <= estimated_size_hint)
+# ifdef _WIN32
+  for (;;) {
+    const hash_defs *hd1 = hd+1 ;
+    if (hd1->mod == 0 || hd1->mod > estimated_size_hint)
+      break ;
     ++ hd ;
+  }
+# else
+  while (hd [1].mod != 0 && hd [1].mod <= estimated_size_hint)
+    ++ hd ;
+# endif
 
   h = XMALLOC (sizeof (hlst) + (hd->mod - 1) * sizeof (void*));
   h->z          =   *hd ;
@@ -192,8 +201,17 @@ copy_hlst
 
   else if (estimated_size_hint != h->z.mod)
     /* find appropriate list size, will stop at the last entry */
-    while (hd [1].mod && hd [1].mod <= estimated_size_hint)
+# ifdef _WIN32
+    for (;;) {
+      const hash_defs *hd1 = hd+1 ;
+      if (hd1->mod == 0 || hd1->mod > estimated_size_hint)
+        break ;
       ++ hd ;
+    }
+# else
+    while (hd [1].mod != 0 && hd [1].mod <= estimated_size_hint)
+      ++ hd ;
+# endif
 
   new = (copy_only = (hd->mod == h->z.mod)) == 0
 
@@ -709,7 +727,7 @@ hlst_statistics
   }
 
   sum = hs.buckets.sum.entries + hs.buckets.idle ;
-  fprintf (stderr, "Buckets: %u out of %u are busy, min/max fill: %u/\n",  
+  fprintf (stderr, "Buckets: %u out of %u are busy, min/max fill: %u/%u\n",  
 	   hs.buckets.busy, sum, hs.fill.min, hs.fill.max);
   
   /* busy buckets */
