@@ -29,6 +29,10 @@
 /* want version stuff */
 #include "libvers.h"
 
+char *nessuslib_version()
+{
+  return strdup(VERSION);
+}
 ExtFunc
 void nessus_lib_version(major, minor, rev)
  int * major, *minor, *rev;
@@ -56,6 +60,38 @@ int nessuslib_pthreads_enabled()
 }
 #endif
 
+
+ExtFunc
+void plug_set_id(desc, id)
+ struct arglist * desc;
+ int id;
+{
+ arg_add_value(desc, "ID", ARG_INT, sizeof(int), (void*)id);
+}
+
+ExtFunc
+void plug_set_cve_id(desc, id)
+ struct arglist * desc;
+ char * id;
+{
+ arg_add_value(desc, "CVE_ID", ARG_STRING, strlen(id), estrdup(id));
+}
+
+ExtFunc 
+void plug_set_see_also(desc, url)
+ struct arglist * desc;
+ char * url;
+{
+ struct arglist * seealso = arg_get_value(desc, "SEE_ALSO");
+ if(!seealso)
+ {
+  seealso = emalloc(sizeof(struct arglist *));
+  arg_add_value(desc, "SEE_ALSO",  ARG_ARGLIST, -1, seealso);
+ }
+ arg_add_value(seealso, "url", ARG_STRING, strlen(url), estrdup(url));
+}
+
+
 ExtFunc
 void plug_set_family(desc, family, language)
  struct arglist * desc; 
@@ -145,6 +181,32 @@ void plug_require_port(desc, portname)
 
  }
 }
+
+ExtFunc 
+void plug_require_udp_port(desc, portname)
+ struct arglist * desc;
+ const char * portname;
+{
+ struct arglist * ports;
+ int iport = atoi(portname);
+ 
+ if(portname)
+ {
+  ports = arg_get_value(desc, "required_udp_ports");
+  if(!ports)
+  {
+   ports = emalloc(sizeof(struct arglist));
+   arg_add_value(desc, "required_udp_ports", ARG_ARGLIST, -1, ports);
+  }
+  
+
+  if(iport)
+   arg_add_value(ports, portname, ARG_INT, sizeof(iport), (char*)iport);
+  else 
+   arg_add_value(ports, portname, ARG_STRING, strlen(portname), strdup(portname));
+
+ }
+}
  
 
 
@@ -395,21 +457,30 @@ int host_get_port_state_proto(plugdata, portnum, proto)
  int portnum;
  char * proto;
 { 
- struct arglist * hostdata = arg_get_value(plugdata, "HOSTNAME");
- struct arglist * ports;
  char * port_s;
  unsigned short * range;
- int i,flag;
+ int flag;
  int num;
 
 
+ if(!proto)
+  proto = "tcp";
+  
  /* Check that we actually scanned the port */
- if(!plug_get_key(plugdata, "Host/scanned")){
+ 
+ if(!strcmp(proto, "tcp") && !plug_get_key(plugdata, "Host/scanned")){
 #ifdef DEBUG
 	printf("HOST NOT SCANNED\n");
 #endif
 	return(1);
 	}
+
+ else if(!strcmp(proto, "udp") && !plug_get_key(plugdata, "Host/udp_scanned"))
+       {
+        return 1;
+      }
+      
+      		
  range = arg_get_value(plugdata, "ports");
  if(!range){
 #ifdef DEBUG
@@ -474,13 +545,14 @@ struct in_addr * plug_get_host_ip(desc)
  else return(NULL);
 }
 
+/* Pluto 24.6.00: reduced to one, and left the orig in place */
 
-ExtFunc
-void proto_post_hole(desc, port, proto, action)
+void proto_post_wrapped(desc, port, proto, action, what)
  struct arglist * desc;
  int port;
  const char * proto;
  const char * action;
+ const char * what;
 {
 
  char *t;
@@ -492,20 +564,24 @@ void proto_post_hole(desc, port, proto, action)
  char * naction;
  int len;
  ntp_caps* caps = arg_get_value(desc, "NTP_CAPS");
- 
+ char * cve;
  
  if(!action)
  {
   action = arg_get_value(desc, "DESCRIPTION");
  }
+  cve = arg_get_value(desc, "CVE_ID");
  if(!action)return;
  len = strlen(action);
+ if(cve)len+=strlen(cve)+20;
  c = (char)13;
  
  if(!caps)return;
+
  hostdata = arg_get_value(desc, "HOSTNAME");
  naction = emalloc(len+1);
- strncpy(naction, action, len);
+ if(cve)sprintf(naction, "%s\nCVE : %s\n", action, cve);
+ else strncpy(naction, action, len);
  while((t=strchr(naction, '\n'))||(t=strchr(naction, c)))t[0]=';';
  buffer=malloc(1024+len);
  if(caps->ntp_11) {
@@ -533,20 +609,22 @@ void proto_post_hole(desc, port, proto, action)
      *idbuffer = '\0';
  }
   if(port>0){
-      char * asc_port = malloc(20);
      sprintf(buffer,
-	     "SERVER <|> HOLE <|> %s <|> %s (%d/%s) <|> %s %s<|> SERVER\n",
+	     "SERVER <|> %s <|> %s <|> %s (%d/%s) <|> %s %s<|> SERVER\n",
+	     what,
   	(char *)arg_get_value(hostdata, "FQDN"),
         service ? service->s_name:"unknown",
 	     port, proto, naction, idbuffer);
     
    } else
      sprintf(buffer,
-	     "SERVER <|> HOLE <|> %s <|> general/%s <|> %s %s<|> SERVER\n",
+	     "SERVER <|> %s <|> %s <|> general/%s <|> %s %s<|> SERVER\n",
+	     what,
   	(char *)arg_get_value(hostdata, "FQDN"), 
 	     proto, naction, idbuffer);
  } else {
-   sprintf(buffer, "SERVER <|> HOLE <|> %s <|> %d:%s <|> SERVER\n", 
+   sprintf(buffer, "SERVER <|> %s <|> %s <|> %d:%s <|> SERVER\n", 
+	   what,
 	   (char *)arg_get_value(hostdata, "FQDN"), port, naction);
  }
  soc = (int)arg_get_value(desc, "SOCKET");
@@ -559,6 +637,19 @@ void proto_post_hole(desc, port, proto, action)
  return;
 }
 
+/* Pluto end */
+
+ExtFunc
+void proto_post_hole(desc, port, proto, action)
+ struct arglist * desc;
+ int port;
+ const char * proto;
+ const char * action;
+{
+  proto_post_wrapped(desc, port, proto, action, "HOLE");
+  return;
+}
+
 
 ExtFunc
 void post_hole(desc, port, action)
@@ -607,160 +698,46 @@ void proto_post_info(desc, port, proto, action)
  const char * proto;
  const char * action;
 {
- char *t;
- char * buffer;
- int soc;
- register struct arglist * hostdata;
- char c;
- char * naction;
- struct arglist * globs;
- int len;
- ntp_caps* caps = arg_get_value(desc, "NTP_CAPS");
+  proto_post_wrapped(desc, port, proto, action, "INFO");
+  return;
+}
  
+ExtFunc
+void post_note(desc, port, action)
+ struct arglist * desc;
+ int port;
+ const char * action;
+{
+  proto_post_note(desc, port, "tcp", action);
+} 
 
- c = (char)13;
- if(!caps)return;
- hostdata = arg_get_value(desc, "HOSTNAME");
- if(!action)
- {
-  action = arg_get_value(desc, "DESCRIPTION");
- }
- if(!action)return;
- len = strlen(action);
- 
- naction = emalloc(len+1);
- strncpy(naction, action, len);
- while((t=strchr(naction, '\n'))||(t=strchr(naction, c)))t[0]=';';
- buffer=malloc(1024+len);
- if(caps->ntp_11) {
-  struct servent * service = NULL;
-   char idbuffer[32];
-   if (caps->scan_ids) {
-     /* XXX
-      * This check can later be removed when we can trust in the ID being
-      * present. It is mainly because I'm not sufficiently used to the
-      * Nessus sources.
-      *
-      * 29.01.1999, Jochen Wiedmann
-      */
-     if (arg_get_type(desc, "ID") == -1) {
-       *idbuffer = '\0';
-     } else {
-       int id = (int)arg_get_value(desc, "ID");
-       sprintf(idbuffer, "<|> %d ", id);
-     }
-   } else {
-     *idbuffer = '\0';
-   }
-  if(port>0){
-        char * asc_port = malloc(20);
      
-  	service = getservbyport(htons((unsigned short)port), proto);
-#ifndef NESSUSNT
-	endservent();
-#endif
-        sprintf(buffer,
-	     "SERVER <|> INFO <|> %s <|> %s (%d/%s) <|> %s %s<|> SERVER\n",
-  	(char *)arg_get_value(hostdata, "FQDN"),
-        service ? service->s_name:"unknown",
-	     port, proto, naction, idbuffer);
+ExtFunc
+void post_note_udp(desc, port, action)
+ struct arglist * desc;
+ int port;
+ const char * action;
+{
+ proto_post_note(desc, port, "udp", action);
+}
 	   
-   } else
-     sprintf(buffer,
-	     "SERVER <|> INFO <|> %s <|> general/%s <|> %s %s<|> SERVER\n",
-	     (char *)arg_get_value(hostdata, "FQDN"), proto, naction,
-	     idbuffer);
- } else {
-   sprintf(buffer, "SERVER <|> INFO <|> %s <|> %d:%s <|> SERVER\n", 
-	   (char *)arg_get_value(hostdata, "FQDN"), port, naction);
- }
- soc = (int)arg_get_value(desc, "SOCKET");
- globs = emalloc(sizeof(struct arglist));
- arg_add_value(globs, "global_socket", ARG_INT, sizeof(int), (void *)soc);
-
- auth_send(globs, buffer);
- efree(&buffer);
- arg_free_all (globs);
- /* efree(&globs); */
- efree(&naction);
- return;
-} 
-
 
 ExtFunc
-void l_post_hole(desc, port, action)
+void proto_post_note(desc, port, proto, action)
  struct arglist * desc;
- const char * port;
+ int port;
+ const char * proto;
  const char * action;
 {
- struct arglist * ports;
- struct arglist * pport;
- struct arglist * report;
- char * comm;
- 
- if(!action)return;
- comm = emalloc(strlen(action)+1);
- strncpy(comm, action, strlen(action));
- ports = arg_get_value(desc, "PORTS");
- if(!ports)
- {
-  ports = emalloc(sizeof(struct arglist));
-  arg_add_value(desc, "PORTS", ARG_ARGLIST, -1, ports);
- }
- pport = arg_get_value(ports, port);
- 
- if(!pport){
- 	pport = emalloc(sizeof(struct arglist));
- 	arg_add_value(ports, port, ARG_ARGLIST, -1, pport);
- 	arg_add_value(pport, "STATE", ARG_INT, sizeof(int),(void *)1);
- 	}
- report = arg_get_value(pport, "REPORT");
- if(!report)
- {
-  report = emalloc(sizeof(struct arglist));
-  arg_add_value(pport, "REPORT", ARG_ARGLIST, -1, report);
- }
- while(report && report->next)report = report->next;
- arg_add_value(report, "HOLE", ARG_STRING, strlen(comm), comm);
- return;
-} 
-
-
-ExtFunc
-void l_post_info(desc, port, action)
- struct arglist * desc;
- const char * port;
- const char * action;
-{
- struct arglist * ports;
- struct arglist * pport;
- struct arglist * report;
- char * comm;
- 
- if(!action)return;
- comm = emalloc(strlen(action)+1);
- strncpy(comm, action, strlen(action));
- ports = arg_get_value(desc, "PORTS");
- if(!ports)
- {
-  ports = emalloc(sizeof(struct arglist));
-  arg_add_value(desc, "PORTS", ARG_ARGLIST, -1, ports);
- }
- pport = arg_get_value(ports, port);
- 
- if(!pport){
- 	pport = emalloc(sizeof(struct arglist));
- 	arg_add_value(ports, port, ARG_ARGLIST, -1, pport);
- 	arg_add_value(pport, "STATE", ARG_INT, sizeof(int),(void *)1);
- 	}
- report = arg_get_value(pport, "INFO");
- if(!report)
- {
-  report = emalloc(sizeof(struct arglist));
-  arg_add_value(pport, "INFO", ARG_ARGLIST, -1, report);
- }
- while(report && report->next)report = report->next;
- arg_add_value(report, "INFO", ARG_STRING, strlen(comm), comm);
+  /*
+   * Backward compatibility. We only use the notes if the remote
+   * client accepts them
+   */
+  char * allow_notes = get_preference(desc, "ntp_client_accepts_notes");
+  if(allow_notes && !strcmp(allow_notes, "yes")) 
+   proto_post_wrapped(desc, port, proto, action, "NOTE");
+  else
+   proto_post_wrapped(desc, port, proto, action, "INFO");
  return;
 } 
  
@@ -787,11 +764,16 @@ void add_plugin_preference(desc, name, type, defaul)
  struct arglist * prefs = arg_get_value(desc, "preferences");
  char * p_name = arg_get_value(desc, "NAME");
  char * pref;
+ char * cname;
+ 
+ cname = strdup(name);
+ while(cname[strlen(cname)-1]==' ')cname[strlen(cname)-1]='\0';
  
  if(!prefs || !p_name)return;
- pref = emalloc(strlen(p_name)+10+strlen(type)+strlen(name));
- sprintf(pref, "%s[%s]:%s", p_name, type, name);
+ pref = emalloc(strlen(p_name)+10+strlen(type)+strlen(cname));
+ sprintf(pref, "%s[%s]:%s", p_name, type, cname);
  arg_add_value(prefs, pref, ARG_STRING, strlen(defaul), estrdup(defaul));
+ efree(&cname);
  efree(&pref);
 }
 
@@ -803,6 +785,9 @@ char * get_plugin_preference(desc, name)
 {
  struct arglist * prefs = arg_get_value(desc, "preferences");
  char * plug_name = arg_get_value(desc, "NAME");
+ char * cname = strdup(name);
+ while(cname[strlen(cname)-1]==' ')cname[strlen(cname)-1]='\0';
+ 
  while(prefs && prefs->next)
  {
   char * a= NULL, *b = NULL;
@@ -816,10 +801,11 @@ char * get_plugin_preference(desc, name)
   if(c)
   {
    b+=2*sizeof(char);
-   if(!strcmp(name, b)){
+   if(!strcmp(cname, b)){
    	a[0] = 0;
 	if(!strcmp(t, plug_name)){
 		efree(&t);
+		efree(&cname);
 		return(prefs->value);
 		}
 	}
@@ -827,6 +813,7 @@ char * get_plugin_preference(desc, name)
   efree(&t);
   prefs = prefs->next;
  }
+ efree(&cname);
  return(NULL);
 }
 
@@ -847,9 +834,19 @@ void plug_set_key(args, name, type, value)
  if(!name || !value)return;
  switch(type)
  {
+   char * t;
   case ARG_STRING :
    str = emalloc(strlen(name)+strlen(value)+10);
+   t = value;
+   while((t = strchr(t, '\n')))t[0]='';
+   t = value;
+   while((t = strchr(t, '\r')))t[0]='';
+
    sprintf(str, "%d %s=%s;\n", ARG_STRING, name, (char *)value);
+   t = value;
+   while((t = strchr(t, '')))t[0]='\n';
+   t = value;
+   while((t = strchr(t, '')))t[0]='\r';
    break;
   case ARG_INT :
    str = emalloc(strlen(name)+20);
@@ -878,11 +875,13 @@ void plug_set_key(args, name, type, value)
    
    while(sent < len)
    {
-    sent += send(pip, str+sent, len+1, 0);
-    if(sent < 0){
+     int e;
+    e = send(pip, str+sent, len+1, 0);
+    if(e < 0){
     	perror("send ");
 	return;
 	}
+    sent+=e;
    }
   }
 } 
@@ -903,10 +902,12 @@ scanner_add_port(args, port, proto)
  int len;
  int soc;
  struct arglist * globs;
+#ifndef NESSUSNT
+   endservent();
+#endif 
  
  
- 
- host_add_port(args, port, 1);
+ host_add_port_proto(args, port, 1, proto);
  
  len = 255 + (hn ? strlen(hn):0) + 
  	    (serv ?strlen(serv->s_name):strlen("unknown"));
@@ -937,15 +938,60 @@ void * plug_get_key(args, name)
  struct arglist * args;
  char * name;
 {
+#ifndef NESSUSNT
  struct arglist * key = (struct arglist *)arg_get_value(args, "key");
+ int type;
+ 
  if(!key)return(NULL);
- return(arg_get_value(key, name));
+ 
+ type = arg_get_type(key, name);
+ if(type >= 0)
+ {
+  if(type == ARG_STRING)
+     {
+      char * t = arg_get_value(key, name);
+      char * v = t;
+      if(v)
+      {
+       while((v = strchr(v, '')))v[0]='\n';
+       v = t;
+       while((v = strchr(v, '')))v[0]='\r';
+      }
+      
+      return t;
+     }
+   else if(type == ARG_INT)
+    return arg_get_value(key,name);
+  else if(type == ARG_ARGLIST)
+  {
+   struct arglist * value = arg_get_value(key, name);
+   while(value && value->next)
+   {
+     int pid;  
+    if(!(pid = fork()))
+     {
+     arg_set_value(key, name, -1, value->value);
+     arg_set_type(key, name, value->type);
+     return value->value;
+     }
+    else
+     {
+      waitpid(pid,NULL, 0);
+    }
+    value = value->next;
+   }
+   exit(0);
+  }
+ }
+ else return NULL;
+#else
+ return NULL;
+#endif  
 }
 
 ExtFunc unsigned int 
 plug_get_host_open_port(struct arglist * desc)
 {
- int soc;
  struct arglist * h = arg_get_value(desc, "key");
  char * str = "Ports/tcp/";
  char * t;
