@@ -6,7 +6,7 @@
  *
  *       Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *    $Id: ptycall.c,v 1.21 2000/01/07 12:06:12 jordan Exp $ 
+ *    $Id: ptycall.c,v 1.23 2000/06/14 12:32:15 jordan Exp $ 
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -178,11 +178,13 @@
 # ifdef HAVE_TERMIO_H
 #  include <termio.h>
 # else
-#  ifdef HAVE_SGTTY_H
-#   include <sgtty.h>
-#  else
-#   error Neiter termio.h/termios.h nor sgtty.h available
-#  endif
+#  ifdef HAVE_SYS_TERMIO_H
+#   ifdef HAVE_SGTTY_H
+#    include <sgtty.h>
+#   else
+#    error Neiter termio.h/termios.h nor sgtty.h available
+#   endif
+#  endif /* HAVE_SYS_TERMIO_H */
 # endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
@@ -455,7 +457,6 @@ static io_dev*
 master_pty
   (void)
 {
-  int err ;
   io_dev *dev = create_dev ();
   
 # ifdef HAVE_DEV_PTMX
