@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: iostream.c,v 1.5 1999/12/12 16:55:56 jordan Exp $
+ *   $Id: iostream.c,v 1.10 2000/08/17 20:33:59 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -63,8 +63,8 @@
 #endif
 
 #include "messages.h"
-#include "rnd-pool.h"
-#include "make-primes.h"
+#include "rand/rnd-pool.h"
+#include "pubkey/make-primes.h"
 
 #define __IOSTREAM_PRIVATE__ /* prevent from using send/recv auto-replaces */
 #include "iostream.h"
@@ -1012,6 +1012,9 @@ io_ptdestroy
   /* destroy cipher stuff */
   cipher_sema_destroy ();
 
+  /* destroy setup stuff */
+  setup_sema_destroy () ;
+
   /* support for other peks functions */
   __sema_destroy (); 
   
@@ -1074,6 +1077,9 @@ io_ptinit
   /* initialize cipher stuff */
   cipher_sema_create (attr);
 
+  /* init setup stuff */
+  setup_sema_create (attr) ;
+
   io_pt = XMALLOC (sizeof (io_mutex)) ;
   pthread_mutex_init (&io_pt->mutex, attr);
 
@@ -1485,7 +1491,7 @@ io_send
   int r, (*fn) (void *, char *msg, unsigned len, int flags);
   void *desc;
   io_mutex *lockd;
-  
+ 
   SEND_TEXT ("io_send", buf, len, fd);
 
   if ((lockd = __enter_lock_phase_II (fd, 1)) != 0) {
@@ -1567,10 +1573,10 @@ io_recv
     } else {
       /* get the read table descriptor */
       desc = rw_table [fd].rd.desc_ptr ;
-      tmo  = rw_table [fd].recv_tmo ;
       __lock_transition_II_to_III (lockd);
-      /* peep into toe input cache */
-      n = (*ctl) (desc, IO_PENDING_CACHE, 0) ;
+      /* peep into the input cache */
+      if ((n = (*ctl) (desc, IO_PENDING_CACHE, 0)) <= 0)
+	tmo = rw_table [fd].recv_tmo ;
       __release_lock_phase_III (lockd) ;
     }
   }
@@ -1588,7 +1594,7 @@ io_recv
 #   endif /* USE_POLL_TEST */
 
 #   ifdef USE_ALARM_TEST
-    time_t now ;
+    time_t now = 0;
     if (tmo > 0)
       now = time (0);
 #   endif /* USE_ALARM_TEST */
