@@ -6,7 +6,7 @@
  *
  *   Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
  *
- *   $Id: peks-file.c,v 1.15 2000/01/02 18:55:55 jordan Exp $
+ *   $Id: peks-file.c,v 1.20 2000/08/14 21:18:05 jordan Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Library General Public
@@ -76,12 +76,12 @@ typedef struct stat struct_stat ;
 #include "peks-setup.h"
 #include "peks-baseXX.h"
 #include "peks-string.h"
-#include "make-primes.h"
+#include "pubkey/make-primes.h"
 #include "messages.h"
-#include "rnd-pool.h"
-#include "crandom.h"
+#include "rand/rnd-pool.h"
+#include "rand/crandom.h"
 #include "hostnames.h"
-#include "elgamal.h"
+#include "pubkey/elgamal.h"
 #include "baseXX.h"
 
 /* ---------------------------------------------------------------------- *
@@ -344,7 +344,8 @@ peks_private_access
 
 peks_key *
 read_peks_key
-  (const char  *usr,
+  (char       **ACC, /* access mask as given in the tag */
+   const char  *usr,
    const char  *tag,
    int       g_type, /* num val of 2nd entry, or -1 to disable */
    const char *file,
@@ -353,7 +354,8 @@ read_peks_key
 {
   FILE *inp;
   char read_buf [MAX_LINE_LENGTH] ;
-  const char *s ;
+  const char *s, *t ;
+  int n ;
 
   /* open input file if it exists */
   if ((usr == 0 && tag == 0) || file == 0) {
@@ -434,6 +436,19 @@ read_peks_key
 	return 0;
       }
 #     endif /* TRUE_ELG_KEY_BITS */
+
+      /* return the access specs, separately */
+      if (ACC != 0 && (t = strchr (read_buf, ':')) != 0) {
+	/* get start of key specs */
+	if ((s = strchr (read_buf, '@')) == 0)
+	  s = tag ;
+	else
+	  ++ s ;
+	if ((n = t - s) > 0) {
+	  *ACC = memcpy (vmalloc (n + 1), s, n) ;
+	  (*ACC) [n] = '\0' ;
+	}
+      }
       return k;
     }
   }
@@ -701,7 +716,7 @@ peks_write_passwd
 
   POINT_OF_RANDOM_STACK (5);
 
-  if (passwd == 0 || k == 0)  /* delete */
+  if (passwd == 0 || k == 0) /* delete */
     return save_peks_key (0, usr, host, 0, file, 0, 0) ;
 
   s        = ALLOCA (sizeof (peks_key)) ;
@@ -750,10 +765,11 @@ peks_read_passwd
   (const peks_key *k,
    const char   *usr,
    const char  *host,
+   int   append_mask,
    const char  *file)
 {
   peks_key *user_pwd ;
-  char *pwd ;
+  char *pwd, *mask = 0;
 # ifdef PASSWD_COMPAT_MODE
   unsigned n;
 # endif
@@ -763,10 +779,18 @@ peks_read_passwd
     return 0 ;
   }
 
-  if ((user_pwd = read_peks_key (usr, host, 0, file, 0, 0)) == 0) {
-    if (errno == 0)
-      errno = errnum (NO_SUCH_USER) ;
-    return 0 ;
+  if ((user_pwd = read_peks_key 
+       (append_mask ? &mask : 0, usr, host, 0, file, 0, 0)) == 0) {
+    if (errno)
+      return 0;
+    if (host == 0 || 
+	/* try without host name, next */
+	(user_pwd = read_peks_key 
+	 (append_mask ? &mask : 0, usr, 0, 0, file, 0, 0)) == 0) {
+      if (errno == 0)
+	errno = errnum (NO_SUCH_USER) ;
+      return 0 ;
+    }
   }
 
 # ifdef PASSWD_COMPAT_MODE
@@ -777,10 +801,23 @@ peks_read_passwd
 # ifdef PASSWD_COMPAT_MODE
   } else {
     pwd = el_gamal_decrypt
-      (&n, &user_pwd->modulus, &user_pwd->private, &k->modulus, &k->private);
+      (&n,
+       (const mpz_t*)&user_pwd->modulus,
+       (const mpz_t*)&user_pwd->private,
+       (const mpz_t*)&k->modulus, &k->private);
   }
 # endif
 
+  if (mask != 0) {
+    char *s = smalloc (strlen (pwd) + strlen (mask) + 2) ;
+    strcpy (s, pwd) ;
+    strcat (s, " ") ;
+    strcat (s, mask) ;
+    xfree  (pwd) ;
+    xfree (mask) ;
+    pwd = s ;
+  }
+
   end_peks_key (user_pwd) ;
   return pwd ;
 }
@@ -850,11 +887,11 @@ check_peks_sender_key
   POINT_OF_RANDOM_VAR (file) ;
 
   /* check, whether the key file exists, already */
-  if ((fkey = read_peks_key (usr, host, -1, file, 0, 0)) == 0 && errno)
+  if ((fkey = read_peks_key (0, usr, host, -1, file, 0, 0)) == 0 && errno)
     goto done ;
   /* re-check for a user entry without host spec */
   if (fkey == 0 && usr != 0)
-    if ((fkey = read_peks_key (usr, 0, -1, file, 0, 0)) == 0 && errno)
+    if ((fkey = read_peks_key (0, usr, 0, -1, file, 0, 0)) == 0 && errno)
       goto done ;
 
   POINT_OF_RANDOM_VAR (fkey) ;
@@ -930,7 +967,7 @@ read_peks_keyfile
   init_random_gen ((char*)&key, sizeof (key)) ;
 
   /* search for "<usr>@<host>: " first and "<usr>@ ", next */
-  if ((key = read_peks_key (usr, host, -1, file, get_pwd, data)) == 0) {
+  if ((key = read_peks_key (0, usr, host, -1, file, get_pwd, data)) == 0) {
     switch (errno) {
     case errnum (WRONG_PASSPHRASE):
     case errnum (PASSPHRASE_WANTED):
@@ -938,7 +975,7 @@ read_peks_keyfile
       if (s != 0) XFREE (s);
       return 0;
     }
-    if ((key = read_peks_key (usr, 0, -1, file, get_pwd, data)) == 0)
+    if ((key = read_peks_key (0, usr, 0, -1, file, get_pwd, data)) == 0)
       switch (errno) {
       case errnum (WRONG_PASSPHRASE):
       case errnum (PASSPHRASE_WANTED):
@@ -1185,11 +1222,12 @@ char *
 peks_edit_passwd_file
   (const peks_key *k,
    const char  *uath,
-   unsigned    count,
+   int         count,
    const char   *pwd,
    const char  *file)
 {
-  char *host, *usr, *s ;
+  char *host, *usr, *t ;
+  char *s = 0;
 
   if (uath == 0) {
     errno = errnum (NO_USER_NAME) ;
@@ -1204,16 +1242,26 @@ peks_edit_passwd_file
     host ++ ;
   }
 
+  /* prepend by HOME variable */
+  if (home_relative (file)) {
+    if ((s = peks_get_homedir (file + 2)) == 0)
+      return 0;
+    POINT_OF_RANDOM_VAR (s) ;
+    file = s ;
+  }
+
   if ((k == 0 && pwd == 0) ||   /* delete request */
       (k != 0 && pwd != 0))     /* edit request */
-    s = peks_write_passwd 
-      (k, usr, host, (char)count, pwd, file) < 0 ? 0 : (char*)"" ;
-  else
-    /* view request */
-    s = peks_read_passwd (k, usr, host, file) ;
+    t = peks_write_passwd 
+      (k, usr, host, count & 0x1f, pwd, file) < 0 ? 0 : (char*)"" ;
+  else 
+    /* view request, append netmask if count == -1 */
+    t = peks_read_passwd (k, usr, host, count < 0, file) ;
 
+  if (s != 0)
+    XFREE (s);
   DEALLOCA (usr) ;
-  return s ;
+  return t ;
 }
 
 
@@ -1249,7 +1297,7 @@ peks_delete_userkey
     host ++ ;
   }
 
-  if ((k = read_peks_key (usr, host, -1, file, __get_pwd, get_pwd)) == 0) {
+  if ((k = read_peks_key (0, usr, host, -1, file, __get_pwd, get_pwd)) == 0) {
     n = -1 ;
     switch (errno) {
     case errnum (PASSPHRASE_WANTED):
@@ -1288,7 +1336,7 @@ peks_delete_hostkey
     file = s ;
   }
 
-  if ((k = read_peks_key (0, host, -1, file, __get_pwd, get_pwd)) == 0) {
+  if ((k = read_peks_key (0, 0, host, -1, file, __get_pwd, get_pwd)) == 0) {
     if (s != 0)
       XFREE (s);
     switch (errno) {
